@@ -29,18 +29,17 @@
 #define DBUS_API_SUBJECT_TO_CHANGE
 #endif
 
-#include "dbus/dbus-glib.h"
-#include "dbus/dbus-glib-lowlevel.h"
-#include "dbus/dbus.h"
-#include <libtelepathy/tp-constants.h>
-#include <libtelepathy/tp-conn.h>
-#include <libtelepathy/tp-chan.h>
 #include <sys/types.h>
 #include <unistd.h>
 
-#define MISSION_CONTROL_SERVICE "org.freedesktop.Telepathy.MissionControl"
-#define MISSION_CONTROL_IFACE "org.freedesktop.Telepathy.MissionControl"
-#define MISSION_CONTROL_PATH "/org/freedesktop/Telepathy/MissionControl"
+#include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
+#include <dbus/dbus.h>
+#include <libtelepathy/tp-conn.h>
+#include <telepathy-glib/connection.h>
+#include <telepathy-glib/channel.h>
+
+#include <libmissioncontrol/dbus-api.h>
 
 #define MISSIONCONTROL_TYPE       (mission_control_get_type ())
 
@@ -111,8 +110,9 @@ typedef enum {
 
 typedef enum {
 	MC_FILTER_FLAG_INCOMING = 1 << 0,
-	MC_FILTER_FLAG_OUTCOMING = 1 << 1
+	MC_FILTER_FLAG_OUTGOING = 1 << 1
 } McFilterFlag;
+#define MC_FILTER_FLAG_OUTCOMING MC_FILTER_FLAG_OUTGOING
 
 struct _missioncontrol
 {
@@ -208,6 +208,14 @@ McAccount *mission_control_get_account_for_connection (MissionControl *self,
 						       TpConn *connection,
 						       GError **error);
 
+TpConnection *mission_control_get_tpconnection (MissionControl *self,
+						McAccount *account,
+						GError **error);
+
+McAccount *mission_control_get_account_for_tpconnection (MissionControl *self,
+							 TpConnection *connection,
+							 GError **error);
+
 gint mission_control_get_used_channels_count (MissionControl *self,
 					      GQuark type, GError **error);
 
@@ -227,7 +235,7 @@ void mission_control_get_current_status (MissionControl *self,
 void mission_control_free_account_statuses (McAccountStatus *accounts);
 
 gboolean mission_control_remote_avatar_changed (MissionControl *self,
-						TpConn *connection,
+						TpConnection *connection,
 						guint contact_id,
 						const gchar *token,
 						GError **error);
