@@ -34,32 +34,35 @@
  */
 
 #include <glib/gi18n.h>
-#include <libtelepathy/tp-chan-iface-group-gen.h>
-#include <libtelepathy/tp-constants.h>
-#include <libtelepathy/tp-conn.h>
+#include <telepathy-glib/interfaces.h>
 
 #include "mcd-channel.h"
 #include "mcd-enum-types.h"
 
-#define MCD_CHANNEL_PRIV(channel) (G_TYPE_INSTANCE_GET_PRIVATE ((channel), \
-				   MCD_TYPE_CHANNEL, \
-				   McdChannelPrivate))
+#define MCD_CHANNEL_PRIV(channel) (MCD_CHANNEL (channel)->priv)
+#define INVALID_SELF_HANDLE ((guint) -1)
 
 G_DEFINE_TYPE (McdChannel, mcd_channel, MCD_TYPE_MISSION);
 
-typedef struct _McdChannelPrivate
+struct _McdChannelPrivate
 {
     /* Channel info */
-    gchar *channel_object_path;
     gchar *channel_type;
     GQuark channel_type_quark;
     guint channel_handle;
-    TelepathyHandleType channel_handle_type;
+    TpHandleType channel_handle_type;
     gboolean outgoing;
     
     /* Channel created based on the above channel info */
-    TpChan *tp_chan;
-    
+    TpChannel *tp_chan;
+    guint has_group_if : 1;
+
+    /* boolean properties */
+    guint self_handle_ready : 1;
+    guint name_ready : 1;
+    guint local_pending_members_ready : 1;
+    guint inviter_ready : 1;
+
     /* Pending members */
     GArray *pending_local_members;
     gboolean members_accepted;
@@ -68,15 +71,21 @@ typedef struct _McdChannelPrivate
     
     McdChannelStatus status;
     gchar *channel_name;
+    gchar *inviter;
     
     /* Requestor info */
     guint requestor_serial;
     gchar *requestor_client_id;
     
     gboolean is_disposed;
-    
-} McdChannelPrivate;
+};
 
+typedef struct
+{
+    guint member;
+    guint actor;
+} PendingMemberInfo;
+    
 enum _McdChannelSignalType
 {
     STATUS_CHANGED,
@@ -89,14 +98,16 @@ enum _McdChannelPropertyType
     PROP_CONNECTION=1,
     PROP_TP_CHANNEL,
     PROP_CHANNEL_STATUS,
-    PROP_CHANNEL_OBJECT_PATH,
     PROP_CHANNEL_TYPE,
     PROP_CHANNEL_TYPE_QUARK,
     PROP_CHANNEL_HANDLE,
     PROP_CHANNEL_HANDLE_TYPE,
     PROP_OUTGOING,
     PROP_REQUESTOR_SERIAL,
-    PROP_REQUESTOR_CLIENT_ID
+    PROP_REQUESTOR_CLIENT_ID,
+    PROP_SELF_HANDLE_READY,
+    PROP_NAME_READY,
+    PROP_INVITER_READY,
 };
 
 static guint mcd_channel_signals[LAST_SIGNAL] = { 0 };
@@ -105,14 +116,14 @@ static void _mcd_channel_release_tp_channel (McdChannel *channel,
 					     gboolean close_channel);
 
 static void
-on_channel_members_changed (DBusGProxy * group_proxy,
-			    const gchar * message, GArray * added,
-			    GArray * removed, GArray * l_pending,
-			    GArray * r_pending, guint actor,
-			    guint reason, gpointer userdata)
+on_members_changed (TpChannel *proxy, const gchar *message,
+		    const GArray *added, const GArray *removed,
+		    const GArray *l_pending, const GArray *r_pending,
+		    guint actor, guint reason, gpointer user_data,
+		    GObject *weak_object)
 {
-    McdChannel *channel = MCD_CHANNEL (userdata);
-    McdChannelPrivate *priv = MCD_CHANNEL_PRIV (userdata);
+    McdChannel *channel = MCD_CHANNEL (weak_object);
+    McdChannelPrivate *priv = user_data;
     /* Local pending members? Add to the array and exit. */
 
     if (l_pending && l_pending->len > 0)
@@ -121,11 +132,12 @@ on_channel_members_changed (DBusGProxy * group_proxy,
 	/* FIXME: Add duplicity check */
 	for (i = 0; i < l_pending->len; i++)
 	{
-            guint handle;
+	    PendingMemberInfo pmi;
 
-            handle = g_array_index (l_pending, guint, i);
-	    g_array_append_val (priv->pending_local_members, handle);
-	    g_debug ("Added handle %u to channel pending members", handle);
+	    pmi.member = g_array_index (l_pending, guint, i);
+	    pmi.actor = actor;
+	    g_array_append_val (priv->pending_local_members, pmi);
+	    g_debug ("Added handle %u to channel pending members", pmi.member);
 	}
     }
 
@@ -145,8 +157,10 @@ on_channel_members_changed (DBusGProxy * group_proxy,
 
 	    for (j = 0; j < priv->pending_local_members->len; j++)
 	    {
-		if (added_member ==
-		    g_array_index (priv->pending_local_members, guint, i))
+		PendingMemberInfo *pmi;
+
+		pmi = &g_array_index (priv->pending_local_members, PendingMemberInfo, i);
+		if (added_member == pmi->member)
 		{
 		    g_debug
 			("Pending local member added -> do not restore lock");
@@ -183,93 +197,269 @@ on_channel_members_changed (DBusGProxy * group_proxy,
 }
 
 static void
-get_local_pending_cb (DBusGProxy * group_proxy,
-		      GArray * l_pending, GError * error, gpointer userdata)
+inspect_inviter_cb (TpConnection *proxy, const gchar **names, const GError *error,
+		    gpointer user_data, GObject *weak_object)
 {
+    McdChannel *channel = MCD_CHANNEL (weak_object);
+    McdChannelPrivate *priv = user_data;
+
+    if (error)
+	g_warning ("Could not inspect contact handle: %s",
+		   error->message);
+    else
+    {
+	priv->inviter = g_strdup (names[0]);
+	g_debug ("Got inviter: %s", priv->inviter);
+    }
+
+    priv->inviter_ready = TRUE;
+    g_object_notify ((GObject *)channel, "inviter-ready");
+}
+
+
+/**
+ * lookup_actor:
+ *
+ * Find out who invited us: find who is the actor who invited the self_handle,
+ * and inspect it
+ */
+static void
+lookup_actor (McdChannel *channel)
+{
+    McdChannelPrivate *priv = channel->priv;
+    PendingMemberInfo *pmi;
+    gboolean found = FALSE;
+    gint i;
+
+    g_debug ("%s called", G_STRFUNC);
+    for (i = 0; i < priv->pending_local_members->len; i++)
+    {
+	pmi = &g_array_index (priv->pending_local_members, PendingMemberInfo,
+			      i);
+	if (pmi->member == priv->self_handle)
+	{
+	    found = TRUE;
+	    break;
+	}
+    }
+
+    if (found)
+    {
+	GArray request_handles;
+	TpConnection *tp_conn;
+
+	/* FIXME: we should check for
+	 * CHANNEL_GROUP_FLAG_CHANNEL_SPECIFIC_HANDLES and call GetHandleOwners
+	 * if needed. See
+	 * https://sourceforge.net/tracker/index.php?func=detail&aid=1906932&group_id=190214&atid=932444
+	 */
+	request_handles.len = 1;
+	request_handles.data = (gchar *)&pmi->actor;
+	g_object_get (priv->tp_chan, "connection", &tp_conn, NULL);
+	tp_cli_connection_call_inspect_handles (tp_conn, -1,
+						TP_HANDLE_TYPE_CONTACT,
+						&request_handles,
+						inspect_inviter_cb,
+						priv, NULL,
+						(GObject *)channel);
+	g_object_unref (tp_conn);
+    }
+    else
+    {
+	/* couldn't find the inviter, but we have to emit the notification
+	 * anyway */
+	g_debug ("%s: inviter not found", G_STRFUNC);
+	priv->inviter_ready = TRUE;
+	g_object_notify ((GObject *)channel, "inviter-ready");
+    }
+}
+
+static void
+group_get_local_pending_members_with_info (TpChannel *proxy,
+					   const GPtrArray *l_pending,
+					   const GError *error,
+					   gpointer user_data,
+					   GObject *weak_object)
+{
+    McdChannel *channel = MCD_CHANNEL (weak_object);
+    McdChannelPrivate *priv = user_data;
+
+    priv->local_pending_members_ready = TRUE;
     if (error)
     {
 	g_warning ("%s: error: %s", G_STRFUNC, error->message);
-	g_error_free (error);
 	return;
     }
 
     if (l_pending)
     {
-	McdChannelPrivate *priv = MCD_CHANNEL_PRIV (userdata);
 	int i;
 	g_debug ("%u local pending members, adding", l_pending->len);
 	/* FIXME: Add duplicity check */
 	for (i = 0; i < l_pending->len; i++)
 	{
-            guint handle;
+	    PendingMemberInfo pmi;
+	    GValueArray *va;
 
-            handle = g_array_index (l_pending, guint, i);
-	    g_array_append_val (priv->pending_local_members, handle);
-	    g_debug ("Added handle %u to channel pending members", handle);
+	    va = g_ptr_array_index (l_pending, i);
+	    pmi.member = g_value_get_uint (va->values);
+	    pmi.actor = g_value_get_uint (va->values + 1);
+	    g_array_append_val (priv->pending_local_members, pmi);
+	    g_debug ("Added handle %u to channel pending members", pmi.member);
 	}
-	g_array_free (l_pending, TRUE);
+	if (priv->self_handle_ready)
+	    lookup_actor (channel);
     }
 }
 
 /* The callback is called on channel Closed signal */
-void
-on_tp_channel_closed (DBusGProxy * tp_chan, gpointer userdata)
+static void
+on_closed (TpChannel *proxy, gpointer user_data, GObject *weak_object)
 {
-    McdChannel *channel = MCD_CHANNEL (userdata);
-    
+    McdChannel *channel = MCD_CHANNEL (weak_object);
+
+    g_debug ("%s called for %p", G_STRFUNC, channel);
     _mcd_channel_release_tp_channel (channel, FALSE);
     mcd_mission_abort (MCD_MISSION (channel));
     g_debug ("Channel closed");
 }
 
-static void proxy_destroyed (DBusGProxy *tp_chan, McdChannel *channel)
+static void
+proxy_destroyed (TpProxy *self, guint domain, gint code, gchar *message,
+		 gpointer user_data)
 {
-    McdChannelPrivate *priv = MCD_CHANNEL_PRIV (channel);
+    McdChannel *channel = user_data;
 
-    g_debug ("Channel proxy destroyed!");
-    g_object_unref (tp_chan);
+    g_debug ("Channel proxy destroyed (%s)!", message);
+    /*
+    McdChannelPrivate *priv = channel->priv;
+    g_object_unref (priv->tp_chan);
     priv->tp_chan = NULL;
+    */
     mcd_mission_abort (MCD_MISSION (channel));
     g_debug ("Channel closed");
 }
 
+static void
+group_get_self_handle_cb (TpChannel *proxy, guint self_handle,
+			  const GError *error, gpointer user_data,
+			  GObject *weak_object)
+{
+    McdChannel *channel = MCD_CHANNEL (weak_object);
+    McdChannelPrivate *priv = user_data;
+    if (error)
+	g_warning ("get_self_handle failed: %s", error->message);
+    else
+    {
+	priv->self_handle = self_handle;
+	g_debug ("channel %p: got self handle %u", channel, self_handle);
+    }
+    priv->self_handle_ready = TRUE;
+    g_object_notify ((GObject *)channel, "self-handle-ready");
+
+    if (priv->local_pending_members_ready)
+	lookup_actor (channel);
+}
+
+static inline void
+_mcd_channel_setup_group (McdChannel *channel)
+{
+    McdChannelPrivate *priv = channel->priv;
+
+    tp_cli_channel_interface_group_connect_to_members_changed (priv->tp_chan,
+							       on_members_changed,
+							       priv, NULL,
+							       (GObject *)channel,
+							       NULL);
+    tp_cli_channel_interface_group_call_get_self_handle (priv->tp_chan, -1,
+							 group_get_self_handle_cb,
+							 priv, NULL,
+							 (GObject *)channel);
+    tp_cli_channel_interface_group_call_get_local_pending_members_with_info (priv->tp_chan, -1,
+							group_get_local_pending_members_with_info,
+									     priv, NULL,
+									     (GObject *)channel);
+}
+
+static void
+inspect_channel_handle_cb (TpConnection *proxy, const gchar **handle_names,
+			   const GError *error, gpointer user_data,
+			   GObject *weak_object)
+{
+    McdChannel *channel = MCD_CHANNEL (weak_object);
+    McdChannelPrivate *priv = user_data;
+
+    if (error)
+	g_warning ("%s: InspectHandles failed: %s", G_STRFUNC, error->message);
+    else
+	priv->channel_name = g_strdup (handle_names[0]);
+    priv->name_ready = TRUE;
+    g_object_notify ((GObject *)channel, "name-ready");
+}
+
+static void
+_mcd_channel_ready (McdChannel *channel)
+{
+    McdChannelPrivate *priv = channel->priv;
+    TpConnection *tp_conn;
+    GArray request_handles;
+
+    g_object_get (priv->tp_chan,
+		  "connection", &tp_conn,
+		  "handle", &priv->channel_handle,
+		  "handle-type", &priv->channel_handle_type,
+		  NULL);
+    g_debug ("%s: handle %u, type %u", G_STRFUNC,
+	     priv->channel_handle_type, priv->channel_handle);
+    if (priv->channel_handle_type != 0)
+    {
+	/* get the name of the channel */
+	request_handles.len = 1;
+	request_handles.data = (gchar *)&priv->channel_handle;
+	tp_cli_connection_call_inspect_handles (tp_conn, -1,
+						priv->channel_handle_type,
+						&request_handles,
+						inspect_channel_handle_cb,
+						priv, NULL,
+						(GObject *)channel);
+    }
+    g_object_unref (tp_conn);
+
+    priv->has_group_if = tp_proxy_has_interface_by_id (priv->tp_chan,
+						       TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP);
+    if (priv->has_group_if)
+	_mcd_channel_setup_group (channel);
+}
+
+static void
+on_channel_ready (TpChannel *tp_chan, GParamSpec *pspec, McdChannel *channel)
+{
+    gboolean ready;
+
+    g_object_get (tp_chan, "channel-ready", &ready, NULL);
+    if (ready)
+	_mcd_channel_ready (channel);
+}
+
 static void
 _mcd_channel_release_tp_channel (McdChannel *channel, gboolean close_channel)
 {
-    DBusGProxy *group_iface;
     McdChannelPrivate *priv = MCD_CHANNEL_PRIV (channel);
     if (priv->tp_chan)
     {
-	GError *error = NULL;
-	
-        g_debug ("%s: getting group_iface", G_STRFUNC);
-	group_iface = tp_chan_get_interface (priv->tp_chan,
-					     TELEPATHY_CHAN_IFACE_GROUP_QUARK);
-	if (group_iface)
-	{
-	    dbus_g_proxy_disconnect_signal (group_iface,"MembersChanged",
-					    G_CALLBACK (on_channel_members_changed),
-					    channel);
-	}
+	g_signal_handlers_disconnect_by_func (priv->tp_chan,
+					      G_CALLBACK (on_channel_ready),
+					      channel);
         
-	dbus_g_proxy_disconnect_signal (DBUS_G_PROXY (priv->tp_chan),
-					"Closed",
-					G_CALLBACK (on_tp_channel_closed),
-					channel);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (priv->tp_chan),
 					      G_CALLBACK (proxy_destroyed),
 					      channel);
 
-	if (close_channel && priv->channel_type_quark != TP_IFACE_QUARK_CHANNEL_TYPE_CONTACT_LIST)
+	if (close_channel && !TP_PROXY (priv->tp_chan)->invalidated &&
+            priv->channel_type_quark != TP_IFACE_QUARK_CHANNEL_TYPE_CONTACT_LIST)
 	{
 	    g_debug ("%s: Requesting telepathy to close the channel", G_STRFUNC);
-	    tp_chan_close (DBUS_G_PROXY (priv->tp_chan), &error);
-	    if (error)
-	    {
-		g_warning ("%s: Request for channel close failed: %s",
-			   G_STRFUNC, error->message);
-		g_error_free (error);
-	    }
+	    tp_cli_channel_call_close (priv->tp_chan, -1, NULL, NULL, NULL, NULL);
 	}
 	/* Destroy our proxy */
 	g_object_unref (priv->tp_chan);
@@ -278,12 +468,34 @@ _mcd_channel_release_tp_channel (McdChannel *channel, gboolean close_channel)
     }
 }
 
+static inline void
+_mcd_channel_setup (McdChannel *channel, McdChannelPrivate *priv)
+{
+    gboolean ready;
+
+    /* check if the channel is ready; if not, connect to its "channel-ready"
+     * signal */
+    g_object_get (priv->tp_chan, "channel-ready", &ready, NULL);
+    if (ready)
+	_mcd_channel_ready (channel);
+    else
+	g_signal_connect (priv->tp_chan, "notify::channel-ready",
+			  G_CALLBACK (on_channel_ready), channel);
+
+    /* We want to track the channel object closes, because we need to do
+     * some cleanups when it's gone */
+    tp_cli_channel_connect_to_closed (priv->tp_chan, on_closed,
+				      priv, NULL, (GObject *)channel,
+				      NULL);
+    g_signal_connect (priv->tp_chan, "invalidated",
+		      G_CALLBACK (proxy_destroyed), channel);
+}
+
 static void
 _mcd_channel_set_property (GObject * obj, guint prop_id,
 			   const GValue * val, GParamSpec * pspec)
 {
-    DBusGProxy *group_iface;
-    TpChan *tp_chan;
+    TpChannel *tp_chan;
     McdChannel *channel = MCD_CHANNEL (obj);
     McdChannelPrivate *priv = MCD_CHANNEL_PRIV (obj);
 
@@ -299,51 +511,15 @@ _mcd_channel_set_property (GObject * obj, guint prop_id,
 	tp_chan = g_value_get_object (val);
 	if (tp_chan)
 	{
-            g_return_if_fail (priv->channel_object_path != NULL);
             g_return_if_fail (priv->channel_type != NULL);
+            g_return_if_fail (priv->channel_handle >= 0);
  
-	    /* FIXME: BUG in libtelepathy */
-	    /* g_return_if_fail (TELEPATHY_IS_CHAN (tp_chan)); */
 	    g_object_ref (tp_chan);
 	}
 	_mcd_channel_release_tp_channel (channel, TRUE);
 	priv->tp_chan = tp_chan;
 	if (priv->tp_chan)
-	{
-	    group_iface = tp_chan_get_interface (priv->tp_chan,
-						 TELEPATHY_CHAN_IFACE_GROUP_QUARK);
-	    if (group_iface)
-	    {
-		GError *error = NULL;
-		/* Setup channel watches */
-		dbus_g_proxy_connect_signal (group_iface, "MembersChanged",
-					     G_CALLBACK (on_channel_members_changed),
-					     channel, NULL);
-		tp_chan_iface_group_get_local_pending_members_async (group_iface,
-								     get_local_pending_cb,
-								     channel);
-		tp_chan_iface_group_get_self_handle (group_iface,
-						    &priv->self_handle, &error);
-		if (error)
-		    g_warning ("get_self_handle failed: %s", error->message);
-	    }
-	    /* We want to track the channel object closes, because we need to do
-	     * some cleanups when it's gone */
-	    dbus_g_proxy_connect_signal (DBUS_G_PROXY (priv->tp_chan), "Closed",
-					 G_CALLBACK (on_tp_channel_closed),
-					 channel, NULL);
-	    g_signal_connect (priv->tp_chan, "destroy",
-			      G_CALLBACK (proxy_destroyed), channel);
-
-	}
-	break;
-    case PROP_CHANNEL_OBJECT_PATH:
-	/* g_return_if_fail (g_value_get_string (val) != NULL); */
-	g_free (priv->channel_object_path);
-	if (g_value_get_string (val) != NULL)
-	    priv->channel_object_path = g_strdup (g_value_get_string (val));
-	else
-	    priv->channel_object_path = NULL;
+	    _mcd_channel_setup (channel, priv);
 	break;
     case PROP_CHANNEL_TYPE:
 	/* g_return_if_fail (g_value_get_string (val) != NULL); */
@@ -401,9 +577,6 @@ _mcd_channel_get_property (GObject * obj, guint prop_id,
     case PROP_TP_CHANNEL:
 	g_value_set_object (val, priv->tp_chan);
 	break;
-    case PROP_CHANNEL_OBJECT_PATH:
-	g_value_set_string (val, priv->channel_object_path);
-	break;
     case PROP_CHANNEL_TYPE:
 	g_value_set_string (val, priv->channel_type);
 	break;
@@ -425,6 +598,15 @@ _mcd_channel_get_property (GObject * obj, guint prop_id,
     case PROP_REQUESTOR_CLIENT_ID:
 	g_value_set_string (val, priv->requestor_client_id);
 	break;
+    case PROP_SELF_HANDLE_READY:
+	g_value_set_boolean (val, priv->self_handle_ready);
+	break;
+    case PROP_NAME_READY:
+	g_value_set_boolean (val, priv->name_ready);
+	break;
+    case PROP_INVITER_READY:
+	g_value_set_boolean (val, priv->inviter_ready);
+	break;
     default:
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
 	break;
@@ -436,11 +618,11 @@ _mcd_channel_finalize (GObject * object)
 {
     McdChannelPrivate *priv = MCD_CHANNEL_PRIV (object);
     
-    g_free (priv->channel_object_path);
     g_free (priv->channel_type);
     g_array_free (priv->pending_local_members, TRUE);
     g_free (priv->requestor_client_id);
     g_free (priv->channel_name);
+    g_free (priv->inviter);
     
     G_OBJECT_CLASS (mcd_channel_parent_class)->finalize (object);
 }
@@ -450,6 +632,7 @@ _mcd_channel_dispose (GObject * object)
 {
     McdChannelPrivate *priv = MCD_CHANNEL_PRIV (object);
    
+    g_debug ("\n\n%s for %p (is disposed = %d)", G_STRFUNC, object, priv->is_disposed);
     if (priv->is_disposed)
 	return;
 
@@ -501,7 +684,7 @@ mcd_channel_class_init (McdChannelClass * klass)
 				     g_param_spec_object ("tp-channel",
 							  _ ("Telepathy Channel Object"),
 							  _ ("Telepathy Channel Object wrapped by it"),
-							  TELEPATHY_CHAN_TYPE,
+							  TP_TYPE_CHANNEL,
                                                           G_PARAM_READWRITE /* |
                                                           G_PARAM_CONSTRUCT_ONLY */));
     g_object_class_install_property (object_class, PROP_CHANNEL_STATUS,
@@ -511,13 +694,6 @@ mcd_channel_class_init (McdChannelClass * klass)
 							MCD_TYPE_CHANNEL_STATUS,
 							MCD_CHANNEL_PENDING,
 							G_PARAM_READWRITE));
-    g_object_class_install_property (object_class, PROP_CHANNEL_OBJECT_PATH,
-				     g_param_spec_string ("channel-object-path",
-							  _ ("Channel dbus object path"),
-							  _ ("DBus Bus name to use by us"),
-							  NULL,
-							  G_PARAM_READWRITE /*|
-							  G_PARAM_CONSTRUCT_ONLY */));
     g_object_class_install_property (object_class, PROP_CHANNEL_TYPE,
 				     g_param_spec_string ("channel-type",
 							  _ ("Channel type"),
@@ -572,25 +748,48 @@ mcd_channel_class_init (McdChannelClass * klass)
 							  _("Requestor client id"),
 							  _("Requestor client id"),
 							  NULL, G_PARAM_READWRITE));
+    g_object_class_install_property (object_class, PROP_SELF_HANDLE_READY,
+				     g_param_spec_boolean ("self-handle-ready",
+						       _("Self handle ready"),
+						       _("Self handle ready"),
+						       FALSE,
+						       G_PARAM_READABLE));
+    g_object_class_install_property (object_class, PROP_NAME_READY,
+				     g_param_spec_boolean ("name-ready",
+						       _("Name ready"),
+						       _("Name ready"),
+						       FALSE,
+						       G_PARAM_READABLE));
+    g_object_class_install_property (object_class, PROP_INVITER_READY,
+				     g_param_spec_boolean ("inviter-ready",
+						       _("Inviter ready"),
+						       _("Inviter ready"),
+						       FALSE,
+						       G_PARAM_READABLE));
 }
 
 static void
 mcd_channel_init (McdChannel * obj)
 {
-    McdChannelPrivate *priv = MCD_CHANNEL_PRIV (obj);
+    McdChannelPrivate *priv;
+   
+    priv = G_TYPE_INSTANCE_GET_PRIVATE (obj, MCD_TYPE_CHANNEL,
+					McdChannelPrivate);
+    obj->priv = priv;
+
+    priv->self_handle = INVALID_SELF_HANDLE;
     priv->pending_local_members = g_array_new (FALSE, FALSE,
-					       sizeof (guint));
+					       sizeof (PendingMemberInfo));
 }
 
 McdChannel *
-mcd_channel_new (TpChan * tp_chan, const gchar *channel_object_path,
+mcd_channel_new (TpChannel * tp_chan,
 		 const gchar *channel_type, guint channel_handle,
-		 TelepathyHandleType channel_handle_type, gboolean outgoing,
+		 TpHandleType channel_handle_type, gboolean outgoing,
 		 guint requestor_serial, const gchar *requestor_client_id)
 {
     McdChannel *obj;
     obj = MCD_CHANNEL (g_object_new (MCD_TYPE_CHANNEL,
-				     "channel-object-path", channel_object_path,
 				     "channel-type", channel_type,
 				     "channel-handle", channel_handle,
 				     "channel-handle-type", channel_handle_type,
@@ -636,7 +835,9 @@ mcd_channel_get_channel_type_quark (McdChannel *channel)
 const gchar *
 mcd_channel_get_object_path (McdChannel *channel)
 {
-    return MCD_CHANNEL_PRIV (channel)->channel_object_path;
+    McdChannelPrivate *priv = MCD_CHANNEL_PRIV (channel);
+
+    return priv->tp_chan ? TP_PROXY (priv->tp_chan)->object_path : NULL;
 }
 
 guint
@@ -645,12 +846,13 @@ mcd_channel_get_handle (McdChannel *channel)
     return MCD_CHANNEL_PRIV (channel)->channel_handle;
 }
 
-TelepathyHandleType
+TpHandleType
 mcd_channel_get_handle_type (McdChannel *channel)
 {
     return MCD_CHANNEL_PRIV (channel)->channel_handle_type;
 }
 
+#if 0
 /* Similar to the another helper, but uses the array version (InspectHandles), free with g_strfreev*/
 static gchar **
 _contact_handles_to_strings (TpConn * conn, guint handle_type,
@@ -671,10 +873,15 @@ _contact_handles_to_strings (TpConn * conn, guint handle_type,
 
     return contact_addresses;
 }
+#endif
 
 GPtrArray*
 mcd_channel_get_members (McdChannel *channel)
 {
+#if 1
+    g_warning ("%s called, but shouldn't!", G_STRFUNC);
+    return NULL;
+#else
     GObject *connection;
     TpConn *tp_connection;
     GPtrArray *members = NULL; 
@@ -791,14 +998,17 @@ err_missing_interface:
     g_object_unref (connection);
     g_object_unref (tp_connection);
     return members;
+#endif
 }
 
+#if 0
 static inline void
 tp_chan_iface_group_remove_members_with_reason_no_reply (DBusGProxy *proxy, const GArray* IN_contacts, const char * IN_message, const guint IN_reason)
 
 {
     dbus_g_proxy_call_no_reply (proxy, "RemoveMembersWithReason", dbus_g_type_get_collection ("GArray", G_TYPE_UINT), IN_contacts, G_TYPE_STRING, IN_message, G_TYPE_UINT, IN_reason, G_TYPE_INVALID);
 }
+#endif
 
 /**
  * mcd_channel_get_name:
@@ -814,41 +1024,42 @@ mcd_channel_get_name (McdChannel *channel)
 {
     McdChannelPrivate *priv = MCD_CHANNEL_PRIV (channel);
 
-    if (!priv->channel_name)
-    {
-	GObject *connection = NULL;
-	TpConn *tp_conn = NULL;
-	GArray *request_handles;
-	gchar **handle_names = NULL;
-	GError *error = NULL;
-
-	request_handles = g_array_new (FALSE, FALSE, sizeof (guint));
-	g_array_append_val (request_handles, priv->channel_handle);
-
-	g_object_get (channel, "connection", &connection, NULL);
-	g_object_get (connection, "tp-connection", &tp_conn, NULL);
-	if (!tp_conn_inspect_handles (DBUS_G_PROXY (tp_conn),
-				      priv->channel_handle_type,
-				      request_handles,
-				      &handle_names, &error))
-	{
-	    g_warning ("%s: InspectHandles failed: %s",
-		       G_STRFUNC, error->message);
-	    g_error_free (error);
-	}
-	else
-	    priv->channel_name = handle_names[0];
-
-	g_object_unref (connection);
-	g_object_unref (tp_conn);
-	g_array_free (request_handles, TRUE);
-	/* free only the pointer array, not the string itself */
-	g_free (handle_names);
-    }
-
     return priv->channel_name;
 }
 
+/**
+ * mcd_channel_get_inviter:
+ * @channel: the #McdChannel.
+ *
+ * Get the address of the inviter (i.e. the actor who put us in the pending
+ * local members list).
+ *
+ * Returns: a const string holding the inviter address.
+ */
+const gchar *
+mcd_channel_get_inviter (McdChannel *channel)
+{
+    McdChannelPrivate *priv = MCD_CHANNEL_PRIV (channel);
+
+    return priv->inviter;
+}
+
+/**
+ * mcd_channel_get_self_handle:
+ * @channel: the #McdChannel.
+ *
+ * Gets the self handle (the "self-handle-ready" property tells if this datum
+ * is available).
+ *
+ * Returns: the self handle.
+ */
+guint
+mcd_channel_get_self_handle (McdChannel *channel)
+{
+    g_return_val_if_fail (MCD_IS_CHANNEL (channel), INVALID_SELF_HANDLE);
+    return channel->priv->self_handle;
+}
+
 /**
  * mcd_channel_is_missed:
  * @channel: the #McdChannel.
@@ -875,8 +1086,12 @@ mcd_channel_is_missed (McdChannel *channel)
  */
 gboolean
 mcd_channel_leave (McdChannel *channel, const gchar *message,
-		   TelepathyChannelGroupChangeReason reason)
+		   TpChannelGroupChangeReason reason)
 {
+#if 1
+    g_warning ("%s called, but shouldn't!", G_STRFUNC);
+    return FALSE;
+#else
     McdChannelPrivate *priv = MCD_CHANNEL_PRIV (channel);
     DBusGProxy *group;
     GArray members;
@@ -891,5 +1106,6 @@ mcd_channel_leave (McdChannel *channel, const gchar *message,
     tp_chan_iface_group_remove_members_with_reason_no_reply (group, &members,
 							     message, reason);
     return TRUE;
+#endif
 }
 
