@@ -35,8 +35,6 @@
 
 #include <dlfcn.h>
 #include <glib/gi18n.h>
-#include <libtelepathy/tp-chan-iface-group-gen.h>
-#include <libtelepathy/tp-ch-gen.h>
 
 #include "mcd-signals-marshal.h"
 #include <libmissioncontrol/mc-account.h>
@@ -86,7 +84,7 @@ typedef struct _McdDispatcherPrivate
     GSList *filter_dlhandles;
     gchar *plugin_dir;
     GData *interface_filters;
-    DBusGConnection *dbus_connection;
+    TpDBusDaemon *dbus_daemon;
 
     /* Channel handlers */
     GHashTable *channel_handler_hash;
@@ -117,7 +115,7 @@ enum
 {
     PROP_0,
     PROP_PLUGIN_DIR,
-    PROP_DBUS_CONNECTION,
+    PROP_DBUS_DAEMON,
     PROP_MCD_MASTER,
 };
 
@@ -133,6 +131,28 @@ enum _McdDispatcherSignalType
 static guint mcd_dispatcher_signals[LAST_SIGNAL] = { 0 };
 
 static void mcd_dispatcher_context_free (McdDispatcherContext * ctx);
+typedef void (*tp_ch_handle_channel_reply) (DBusGProxy *proxy, GError *error, gpointer userdata);
+
+static void
+tp_ch_handle_channel_async_callback (DBusGProxy *proxy, DBusGProxyCall *call, void *user_data)
+{
+  DBusGAsyncData *data = (DBusGAsyncData*) user_data;
+  GError *error = NULL;
+  dbus_g_proxy_end_call (proxy, call, &error, G_TYPE_INVALID);
+  (*(tp_ch_handle_channel_reply)data->cb) (proxy, error, data->userdata);
+  return;
+}
+
+static inline DBusGProxyCall*
+tp_ch_handle_channel_async (DBusGProxy *proxy, const char * IN_Bus_Name, const char* IN_Connection, const char * IN_Channel_Type, const char* IN_Channel, const guint IN_Handle_Type, const guint IN_Handle, tp_ch_handle_channel_reply callback, gpointer userdata)
+
+{
+  DBusGAsyncData *stuff;
+  stuff = g_new (DBusGAsyncData, 1);
+  stuff->cb = G_CALLBACK (callback);
+  stuff->userdata = userdata;
+  return dbus_g_proxy_begin_call (proxy, "HandleChannel", tp_ch_handle_channel_async_callback, stuff, g_free, G_TYPE_STRING, IN_Bus_Name, DBUS_TYPE_G_OBJECT_PATH, IN_Connection, G_TYPE_STRING, IN_Channel_Type, DBUS_TYPE_G_OBJECT_PATH, IN_Channel, G_TYPE_UINT, IN_Handle_Type, G_TYPE_UINT, IN_Handle, G_TYPE_INVALID);
+}
 
 static inline DBusGProxyCall *
 tp_ch_handle_channel_2_async (DBusGProxy *proxy,
@@ -631,12 +651,10 @@ _mcd_dispatcher_handle_channel_async_cb (DBusGProxy * proxy, GError * error,
      * nothing. Thus, we'll try to handle the death */
     
     {
-	McdConnection *connection;
 	DBusGConnection *dbus_connection;
 	GError *unique_proxy_error = NULL;
 	
-	connection = mcd_dispatcher_context_get_connection (context);
-        g_object_get (G_OBJECT (connection), "dbus-connection", &dbus_connection, NULL);
+	dbus_connection = TP_PROXY (priv->dbus_daemon)->dbus_connection;
 	
 	DBusGProxy *unique_name_proxy =
 	    dbus_g_proxy_new_for_name_owner (dbus_connection,
@@ -705,13 +723,13 @@ _mcd_dispatcher_start_channel_handler (McdDispatcherContext * context)
     {
 	struct cancel_call_data *call_data;
 	DBusGProxyCall *call;
-	TpConn *tp_conn;
+	TpConnection *tp_conn;
 	
 	const McdConnection *connection = mcd_dispatcher_context_get_connection (context);
 	DBusGConnection *dbus_connection;
 
+	dbus_connection = TP_PROXY (priv->dbus_daemon)->dbus_connection;
         g_object_get (G_OBJECT (connection),
-                      "dbus-connection", &dbus_connection,
                       "tp-connection", &tp_conn, NULL);
 
 	DBusGProxy *handler_proxy = dbus_g_proxy_new_for_name (dbus_connection,
@@ -723,7 +741,7 @@ _mcd_dispatcher_start_channel_handler (McdDispatcherContext * context)
 		 " obj_path = %s, handle_type = %d, handle = %d",
 		 chandler->bus_name,
 		 chandler->obj_path,
-		 dbus_g_proxy_get_path (DBUS_G_PROXY (tp_conn)),
+		 TP_PROXY (tp_conn)->object_path,
 		 mcd_channel_get_channel_type (channel),
 		 mcd_channel_get_object_path (channel),
 		 mcd_channel_get_handle_type (channel),
@@ -743,11 +761,9 @@ _mcd_dispatcher_start_channel_handler (McdDispatcherContext * context)
 	    options = g_hash_table_new (g_str_hash, g_str_equal);
 	    call = tp_ch_handle_channel_2_async (handler_proxy,
 					/*Connection bus */
-					dbus_g_proxy_get_bus_name (DBUS_G_PROXY
-								   (tp_conn)),
+					TP_PROXY (tp_conn)->bus_name,
 					/*Connection path */
-					dbus_g_proxy_get_path (DBUS_G_PROXY
-							       (tp_conn)),
+					TP_PROXY (tp_conn)->object_path,
 					/*Channel type */
 					mcd_channel_get_channel_type (channel),
 					/*Object path */
@@ -764,11 +780,9 @@ _mcd_dispatcher_start_channel_handler (McdDispatcherContext * context)
 	else
 	    call = tp_ch_handle_channel_async (handler_proxy,
 					/*Connection bus */
-					dbus_g_proxy_get_bus_name (DBUS_G_PROXY
-								   (tp_conn)),
+					TP_PROXY (tp_conn)->bus_name,
 					/*Connection path */
-					dbus_g_proxy_get_path (DBUS_G_PROXY
-							       (tp_conn)),
+					TP_PROXY (tp_conn)->object_path,
 					/*Channel type */
 					mcd_channel_get_channel_type (channel),
 					/*Object path */
@@ -978,7 +992,6 @@ _mcd_dispatcher_set_property (GObject * obj, guint prop_id,
 {
     McdDispatcher *dispatcher = MCD_DISPATCHER (obj);
     McdDispatcherPrivate *priv = MCD_DISPATCHER_PRIV (obj);
-    DBusGConnection *dbus_connection;
     McdMaster *master;
 
     switch (prop_id)
@@ -991,12 +1004,10 @@ _mcd_dispatcher_set_property (GObject * obj, guint prop_id,
 	_mcd_dispatcher_unload_filters (dispatcher);
 	_mcd_dispatcher_load_filters (dispatcher);
 	break;
-    case PROP_DBUS_CONNECTION:
-	dbus_connection = g_value_get_pointer (val);
-	dbus_g_connection_ref (dbus_connection);
-	if (priv->dbus_connection)
-	    dbus_g_connection_unref (priv->dbus_connection);
-	priv->dbus_connection = dbus_connection;
+    case PROP_DBUS_DAEMON:
+	if (priv->dbus_daemon)
+	    g_object_unref (priv->dbus_daemon);
+	priv->dbus_daemon = TP_DBUS_DAEMON (g_value_dup_object (val));
 	break;
     case PROP_MCD_MASTER:
 	master = g_value_get_object (val);
@@ -1026,8 +1037,8 @@ _mcd_dispatcher_get_property (GObject * obj, guint prop_id,
     case PROP_PLUGIN_DIR:
 	g_value_set_string (val, priv->plugin_dir);
 	break;
-    case PROP_DBUS_CONNECTION:
-	g_value_set_pointer (val, priv->dbus_connection);
+    case PROP_DBUS_DAEMON:
+	g_value_set_object (val, priv->dbus_daemon);
 	break;
     case PROP_MCD_MASTER:
 	g_value_set_object (val, priv->master);
@@ -1066,10 +1077,10 @@ _mcd_dispatcher_dispose (GObject * object)
 	priv->master = NULL;
     }
 
-    if (priv->dbus_connection)
+    if (priv->dbus_daemon)
     {
-	dbus_g_connection_unref (priv->dbus_connection);
-	priv->dbus_connection = NULL;
+	g_object_unref (priv->dbus_daemon);
+	priv->dbus_daemon = NULL;
     }
 
     if (priv->channels)
@@ -1152,12 +1163,13 @@ mcd_dispatcher_class_init (McdDispatcherClass * klass)
 							  G_PARAM_READWRITE |
 							  G_PARAM_CONSTRUCT));
     g_object_class_install_property (object_class,
-				     PROP_DBUS_CONNECTION,
-				     g_param_spec_pointer ("dbus-connection",
-							   _("DBus Connection"),
-							   _("DBus connection to use by us"),
-							   G_PARAM_READWRITE |
-							   G_PARAM_CONSTRUCT));
+				     PROP_DBUS_DAEMON,
+				     g_param_spec_object ("dbus-daemon",
+							  _("DBus daemon"),
+							  _("DBus daemon"),
+							  TP_TYPE_DBUS_DAEMON,
+							  G_PARAM_READWRITE |
+							  G_PARAM_CONSTRUCT));
     g_object_class_install_property (object_class,
 				     PROP_MCD_MASTER,
 				     g_param_spec_object ("mcd-master",
@@ -1218,14 +1230,12 @@ mcd_dispatcher_init (McdDispatcher * dispatcher)
 }
 
 McdDispatcher *
-mcd_dispatcher_new (DBusGConnection * dbus_connection, McdMaster * master)
+mcd_dispatcher_new (TpDBusDaemon *dbus_daemon, McdMaster *master)
 {
     McdDispatcher *obj;
     obj = MCD_DISPATCHER (g_object_new (MCD_TYPE_DISPATCHER,
-					"dbus-connection",
-					dbus_connection,
-					"mcd-master", 
-					master, 
+					"dbus-daemon", dbus_daemon,
+					"mcd-master", master, 
 					NULL));
     return obj;
 }
@@ -1292,10 +1302,10 @@ mcd_dispatcher_context_free (McdDispatcherContext * context)
 /* CONTEXT API */
 
 /* Context getters */
-TpChan *
+TpChannel *
 mcd_dispatcher_context_get_channel_object (McdDispatcherContext * ctx)
 {
-    TpChan *tp_chan;
+    TpChannel *tp_chan;
     g_return_val_if_fail (ctx, 0);
     g_object_get (G_OBJECT (ctx->channel), "tp-channel", &tp_chan, NULL);
     g_object_unref (G_OBJECT (tp_chan));
@@ -1323,11 +1333,11 @@ mcd_dispatcher_context_get_connection (McdDispatcherContext * context)
     return connection;
 }
 
-TpConn *
+TpConnection *
 mcd_dispatcher_context_get_connection_object (McdDispatcherContext * ctx)
 {
     const McdConnection *connection;
-    TpConn *tp_conn;
+    TpConnection *tp_conn;
    
     connection = mcd_dispatcher_context_get_connection (ctx); 
     g_object_get (G_OBJECT (connection), "tp-connection",
