@@ -48,9 +48,6 @@
 #include <dbus/dbus-glib-lowlevel.h>
 #include <dbus/dbus.h>
 #include <gconf/gconf-client.h>
-#include <libtelepathy/tp-interfaces.h>
-#include <libtelepathy/tp-constants.h>
-#include <libtelepathy/tp-helpers.h>
 
 #include "mcd-signals-marshal.h"
 #include "mcd-dispatcher.h"
@@ -175,7 +172,8 @@ static gboolean
 mcd_service_get_online_connections (GObject * obj,
 				    gchar *** ret, GError ** error)
 {
-    return mcd_master_get_online_connection_names (MCD_MASTER (obj), ret);
+    mcd_master_get_online_connection_names (MCD_MASTER (obj), ret);
+    return TRUE;
 }
 
 static gboolean
@@ -329,8 +327,8 @@ mcd_service_get_current_status(GObject *obj,
 	McAccount *account = account_node->data;
 	GValue account_data = { 0, };
 	const gchar *name;
-	TelepathyConnectionStatus status;
-	TelepathyConnectionStatusReason reason;
+	TpConnectionStatus status;
+	TpConnectionStatusReason reason;
 	McPresence presence;
 
 	name = mc_account_get_unique_name (account);
@@ -404,7 +402,7 @@ _on_filter_process (DBusGProxy *proxy, guint counter, gboolean process)
 static void
 _on_filter_new_channel (McdDispatcherContext *ctx, DBusGProxy *proxy)
 {
-    TpConn *tp_conn;
+    TpConnection *tp_conn;
     const McdConnection *connection = mcd_dispatcher_context_get_connection (ctx);
     McdChannel *channel = mcd_dispatcher_context_get_channel (ctx); 
     static guint counter = 0;
@@ -416,8 +414,8 @@ _on_filter_new_channel (McdDispatcherContext *ctx, DBusGProxy *proxy)
 
     g_debug ("%s: Filtering new channel", __FUNCTION__);
     dbus_g_proxy_call_no_reply (proxy, "FilterChannel",
-				G_TYPE_STRING, dbus_g_proxy_get_bus_name (DBUS_G_PROXY(tp_conn)),
-				DBUS_TYPE_G_OBJECT_PATH, dbus_g_proxy_get_path (DBUS_G_PROXY(tp_conn)),
+				G_TYPE_STRING, TP_PROXY (tp_conn)->bus_name,
+				DBUS_TYPE_G_OBJECT_PATH, TP_PROXY (tp_conn)->object_path,
 				G_TYPE_STRING, mcd_channel_get_channel_type (channel),
 				DBUS_TYPE_G_OBJECT_PATH, mcd_channel_get_object_path (channel),
 				G_TYPE_UINT, mcd_channel_get_handle_type (channel),
@@ -554,8 +552,8 @@ mcd_register_dbus_object (McdService * obj)
 static void
 _on_account_status_changed (McdPresenceFrame * presence_frame,
 			    McAccount * account,
-			    TelepathyConnectionStatus connection_status,
-			    TelepathyConnectionStatusReason connection_reason,
+			    TpConnectionStatus connection_status,
+			    TpConnectionStatusReason connection_reason,
 			    McdService * obj)
 {
     McPresence presence =
@@ -658,11 +656,11 @@ count_connections (McdOperation *manager, guint *connections)
 
 static void
 _on_status_actual (McdPresenceFrame * presence_frame,
-		   TelepathyConnectionStatus status,
+		   TpConnectionStatus status,
 		   McdService * obj)
 {
     /* Everyone just got disconnected */
-    if (status == TP_CONN_STATUS_DISCONNECTED)
+    if (status == TP_CONNECTION_STATUS_DISCONNECTED)
     {
 	guint connections = 0;
 	/* if there are no connections, then exit. Count the connections
